# Agenten Plattform
#
# (c) 2025 Magnus Bender
#  
# source code released under the terms of GNU Public License Version 3
# https://www.gnu.org/licenses/gpl-3.0.txt

import re

from typing import List

from ums.agent import SolveAgent
from ums.utils import Riddle, RiddleData, RiddleSolution

class CalculatorSolveExpression(SolveAgent):

	def handle(self, riddle: Riddle, data: List[RiddleData]) -> RiddleSolution:
		# remove whitespace
		expression = riddle.question.strip()

		# get all the extracted values
		var_vals = {}
		for d in data:
			e = self.get_extracted(d)
			if not e is None \
				and "variable" in e.other and "value" in e.other \
				and e.other["variable"] in expression:
					var_vals[e.other["variable"]] = e.other["value"]

		# require "=" at the end
		if not expression[-1] == "=":
			return RiddleSolution(solution="Error", explanation="No = at the end of the expression!")
		
		# solve the expression
		#	remove the = and whitespace
		expression = expression[:-1].strip()

		for var, val in var_vals.items():
			# replace the variables by values
			expression = expression.replace(var, str(val))

		# check the expression
		if re.match(r"^[0-9+\-*\/ ]+$", expression) is None:
			return RiddleSolution(solution="Error", explanation="Missing data or faulty expression")

		try:
			# using eval is a usually bad idea, but this is only for demonstration
			#	and expression may only contain "0-9 +-*/"
			result = eval(expression)
		except:
			return RiddleSolution(solution="Error", explanation="Unable to calculate value of expression")
		
		return RiddleSolution(solution=str(result), explanation="{} = {}".format(expression, result))

AGENT_CLASSES = [
	CalculatorSolveExpression
]