# Agenten Plattform
#
# (c) 2025 Magnus Bender
#  
# source code released under the terms of GNU Public License Version 3
# https://www.gnu.org/licenses/gpl-3.0.txt

import re
from typing import List

from ums.agent import GatekeeperAgent
from ums.utils import Riddle, RiddleSolution

class CalculatorCheckResult(GatekeeperAgent):

	def handle(self, solution: List[RiddleSolution], riddle: Riddle) -> List[RiddleSolution]:
		return [self._check_single_solution(s) for s in solution]
	
	def _check_single_solution(self, solution:RiddleSolution) -> RiddleSolution:
		# this is just a simple check, we check if solution and explanation match to the expression 

		match = re.match(r"^([0-9+\-*\/ ]+)\s*=\s*(\d+)$", solution.explanation)
		if match is None:
			return self._update_solution(solution, False, "Invalid math expression")
		
		expression, result = match.group(1), match.group(2)
		if result != solution.solution:
			return self._update_solution(solution, False, "Inconsistent values")
		
		try:
			# using eval is a bad idea, but this is only for demonstration
			#	and expression may only contain "0-9 +-*/"
			own_result = eval(expression)
		except:
			return self._update_solution(solution, False, "Unsolvable expression")
		
		# check the values
		if str(own_result) != solution.solution:
			return self._update_solution(solution, False, "Value of expression does not match solution!")
		else:
			return self._update_solution(solution, True, "Yes, {}".format(solution.explanation))
	
	def _update_solution(self, solution:RiddleSolution, accepted:bool, review:str="") -> RiddleSolution:
		solution.accepted = accepted 
		solution.review = review
		return solution

AGENT_CLASSES = [
	CalculatorCheckResult
]